//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 12.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

namespace gwe.sql {

using System;
using System.IO;
using net.sql;

/**
 * This class provides the mysql implementation of the JDBC ResultSetMetaData
 * interface.
 */
[Serializable]
public class gweMysqlMetaResult : net.sql.ResultSetMetaData {
  /**
   * The result this meta-data exists for.
   */
//  private gweMysql mysql;

  // Data Types
  public const int FIELD_TYPE_DECIMAL     =   0;
  public const int FIELD_TYPE_TINY        =   1;
  public const int FIELD_TYPE_SHORT       =   2;
  public const int FIELD_TYPE_LONG        =   3;
  public const int FIELD_TYPE_FLOAT       =   4;
  public const int FIELD_TYPE_DOUBLE      =   5;
  public const int FIELD_TYPE_NULL        =   6;
  public const int FIELD_TYPE_TIMESTAMP   =   7;
  public const int FIELD_TYPE_LONGLONG    =   8;
  public const int FIELD_TYPE_INT24       =   9;
  public const int FIELD_TYPE_DATE        =  10;
  public const int FIELD_TYPE_TIME        =  11;
  public const int FIELD_TYPE_DATETIME    =  12;
  // Newer data types
  public const int FIELD_TYPE_YEAR        =  13;
  public const int FIELD_TYPE_NEWDATE     =  14;
  public const int FIELD_TYPE_ENUM        = 247;
  public const int FIELD_TYPE_SET         = 248;
  // Older data types
  public const int FIELD_TYPE_TINY_BLOB   = 249;
  public const int FIELD_TYPE_MEDIUM_BLOB = 250;
  public const int FIELD_TYPE_LONG_BLOB   = 251;
  public const int FIELD_TYPE_BLOB        = 252;
  public const int FIELD_TYPE_VAR_STRING  = 253;
  public const int FIELD_TYPE_STRING      = 254;

  private int columnCount;
  private gweMysqlColumn[] cols;

  /**
   * Constructs a new instance of the gweMysqlMetaResult class
   * @param result the gweMysql object
   */
  public gweMysqlMetaResult(gweMysql result) {
//    mysql = result;
    columnCount = result.getColumnCount();
    cols = new gweMysqlColumn[columnCount];
    for(int i = 1; i <= columnCount; i++)
    {
      try {
        cols[i-1] = result.getColumn(i);
      }
      catch(Exception e) {
        Console.WriteLine(e.Message);
        Console.WriteLine(e.StackTrace);
      }
    }
  }

  /**
   * Returns the number of columns in this result set.
   * @see java.sqlResultSetMetaData#getColumnCount
   * @return number of columns
   */
  public int getColumnCount() {
    return columnCount;
//    return mysql.getColumnCount();
  }

  /**
   * Should indicate if field is set AUTO INCREMENT, but no way of
   * getting the information from mysql at the moment.
   * @see java.sql.ResultSetMetaData#isAutoIncrement
   * @return false
   */
  public bool isAutoIncrement(int column) {
    return false;
  }

  /**
   * Column's Case matters to mysql
   * @see java.sql.ResultSetMetaData#isCaseSensitive
   * @return true
   */
  public bool isCaseSensitive(int column) {
    return true;
  }

  /**
   * Can the column be used in a where clause?

   * @see java.sql.ResultSetMetaData#isSearchable
   * @return true unless column is a from of blob
   */
  public bool isSearchable(int column) {
    try {
//      gweMysqlColumn col = mysql.getColumn(column);
      gweMysqlColumn col = cols[column-1];
      return ( !col.isBlob() );
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * Is the column a cash value?
   * @see java.sql.ResultSetMetaData#isCurrency
   * @return false
   */
  public bool isCurrency(int column) {
    return false;
  }

  /**
   * Determines if the column in question is nullable.
   * @see java.sql.ResultSetMetaData#isNullable
   * @exception SQLException thrown if the column is out of range
   * @return true if the column can accept null values, otherwise false
   */
  public int isNullable(int column) {
    try {
//      gweMysqlColumn col = mysql.getColumn(column);
      gweMysqlColumn col = cols[column-1];
      return (col.isNotNull() ? (int)ResultSetMetaData_column.NoNulls :
                                (int)ResultSetMetaData_column.Nullable);
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * Is the column a signed number?
   * @see java.sql.ResultSetMetaData#isSigned
   * @return true
   */
  public bool isSigned(int column) {
    try {
//      gweMysqlColumn col = mysql.getColumn(column);
      gweMysqlColumn col = cols[column-1];
      return (!col.isUnsigned());
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * Gives the display size for this column.
   * @see java.sql.ResultSetMetaData#getColumnDisplaySize
   */
  public int getColumnDisplaySize(int column) {
    try {
//      gweMysqlColumn col = mysql.getColumn(column);
      gweMysqlColumn col = cols[column-1];
      return (col.getLength());
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * This returns table_name.column_name.
   * @see java.sql.ResultSetMetaData#getColumnLabel
   * @param column the column whose label is wanted
   * @return the display value for the field
   */
  public string getColumnLabel(int column) {
    try {
//      gweMysqlColumn col = mysql.getColumn(column);
      gweMysqlColumn col = cols[column-1];
      return (col.getFullName());
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * The name of the requested column.
   * @see java.sql.ResultSetMetaData#getColumnName
   * @exception thrown when the column desired is bad
   * @param column the column whose name is wanted
   * @return the name of the requested column
   */
  public string getColumnName(int column) {
    try {
//      gweMysqlColumn col = mysql.getColumn(column);
      gweMysqlColumn col = cols[column-1];
      return (col.getName());
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * This does not apply to mysql.
   */
  public string getSchemaName(int column) {
    return "";
  }

  /**
   * What's the column's number of decimal digits?
   */
  public int getPrecision(int column) {
    try {
//      gweMysqlColumn col = mysql.getColumn(column);
      gweMysqlColumn col = cols[column-1];
      return (col.getDecimals());
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * What's the column's number of digits ro right of decimal point?
   */
  public int getScale(int column) {
    try {
//      gweMysqlColumn col = mysql.getColumn(column);
      gweMysqlColumn col = cols[column-1];
      return (col.getDecimals());
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * Gives the name of the table to which this column belongs.
   * @see java.sql.ResultSetMetaData#getTableName
   * @param column the column of the field this information is needed for
   * @return the table name
   */
  public string getTableName(int column) {
    try {
//      gweMysqlColumn col = mysql.getColumn(column);
      gweMysqlColumn col = cols[column-1];
      return (col.getTable());
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * mysql does not support this
   */
  public string getCatalogName(int column) {
    return "";
  }

  /**
   * Gives the column type using the types in java.sql.Types.
   * @see java.sqlTypes
   * @see java.sql.ResultSetMetaData#getColumnType
   * @exception SQLException thrown for any number of reasons
   * @param column the column type information is needed on
   * @return the type as listed in java.sql.Types
   */
  public int getColumnType(int column) {
    try {
//      gweMysqlColumn col = mysql.getColumn(column);
      gweMysqlColumn col = cols[column-1];
      switch (col.getType()) {
        case gweMysqlMetaResult.FIELD_TYPE_DECIMAL     : return Types.DECIMAL;
        case gweMysqlMetaResult.FIELD_TYPE_TINY        : return Types.TINYINT;
        case gweMysqlMetaResult.FIELD_TYPE_SHORT       : return Types.SMALLINT;
        case gweMysqlMetaResult.FIELD_TYPE_LONG        : return Types.INTEGER;
        case gweMysqlMetaResult.FIELD_TYPE_FLOAT       : return Types.FLOAT;
        case gweMysqlMetaResult.FIELD_TYPE_DOUBLE      : return Types.DOUBLE;
        case gweMysqlMetaResult.FIELD_TYPE_NULL        : return Types.NULL;
        case gweMysqlMetaResult.FIELD_TYPE_TIMESTAMP   : return Types.TIMESTAMP;
        case gweMysqlMetaResult.FIELD_TYPE_LONGLONG    : return Types.BIGINT;
        case gweMysqlMetaResult.FIELD_TYPE_INT24       : return Types.INTEGER;
        case gweMysqlMetaResult.FIELD_TYPE_DATE        : return Types.DATE;
        case gweMysqlMetaResult.FIELD_TYPE_TIME        : return Types.TIME;
        case gweMysqlMetaResult.FIELD_TYPE_DATETIME    : return Types.TIMESTAMP;
        case gweMysqlMetaResult.FIELD_TYPE_NEWDATE     : return Types.DATE;
        case gweMysqlMetaResult.FIELD_TYPE_ENUM        : return Types.CHAR;
        case gweMysqlMetaResult.FIELD_TYPE_SET         : return Types.CHAR;
        case gweMysqlMetaResult.FIELD_TYPE_TINY_BLOB   : return Types.VARBINARY;
        case gweMysqlMetaResult.FIELD_TYPE_MEDIUM_BLOB : return Types.LONGVARBINARY;
        case gweMysqlMetaResult.FIELD_TYPE_LONG_BLOB   : return Types.LONGVARBINARY;
        case gweMysqlMetaResult.FIELD_TYPE_BLOB        : //return Types.LONGVARBINARY;
          // Thanks for INDEN-san
          if( col.isBinary() ) return Types.LONGVARBINARY;
          return Types.LONGVARCHAR;
        case gweMysqlMetaResult.FIELD_TYPE_VAR_STRING  : return Types.VARCHAR;
        case gweMysqlMetaResult.FIELD_TYPE_STRING      : return Types.CHAR;
        case gweMysqlMetaResult.FIELD_TYPE_YEAR        : return Types.SMALLINT; // YEAR support!!
        default : throw new SQLException("Unknown data type.");
      }
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * Gives the column type as a string.
   * @see java.sql.ResultSetMetaData#getColumnTypeName
   * @exception SQLException thrown randomly for fun
   * @param column the column for which the type name is wanted
   * @return the name of the column type
   */
  public string getColumnTypeName(int column) {
    try {
//      gweMysqlColumn col = mysql.getColumn(column);
      gweMysqlColumn col = cols[column-1];
      switch (col.getType()) {
        case gweMysqlMetaResult.FIELD_TYPE_DECIMAL:
          return "DECIMAL";
        case gweMysqlMetaResult.FIELD_TYPE_TINY:
          return "TINY";
        case gweMysqlMetaResult.FIELD_TYPE_SHORT:
          return "SHORT";
        case gweMysqlMetaResult.FIELD_TYPE_LONG:
          return "LONG";
        case gweMysqlMetaResult.FIELD_TYPE_FLOAT:
          return "FLOAT";
        case gweMysqlMetaResult.FIELD_TYPE_DOUBLE:
          return "DOUBLE";
        case gweMysqlMetaResult.FIELD_TYPE_NULL:
          return "NULL";
        case gweMysqlMetaResult.FIELD_TYPE_TIMESTAMP:
          return "TIMESTAMP";
        case gweMysqlMetaResult.FIELD_TYPE_LONGLONG:
          return "LONGLONG";
        case gweMysqlMetaResult.FIELD_TYPE_INT24:
          return "INT";
        case gweMysqlMetaResult.FIELD_TYPE_DATE:
          return "DATE";
        case gweMysqlMetaResult.FIELD_TYPE_TIME:
          return "TIME";
        case gweMysqlMetaResult.FIELD_TYPE_DATETIME:
          return "DATETIME";
        case gweMysqlMetaResult.FIELD_TYPE_TINY_BLOB:
          return "TINYBLOB";
        case gweMysqlMetaResult.FIELD_TYPE_MEDIUM_BLOB:
          return "MEDIUMBLOB";
        case gweMysqlMetaResult.FIELD_TYPE_LONG_BLOB:
          return "LONGBLOB";
        case gweMysqlMetaResult.FIELD_TYPE_BLOB:
//          return "BLOB";
          // Thanks for INDEN-san
          if( col.isBinary() ) return "LONGBLOB";
          return "LONGTEXT";
        case gweMysqlMetaResult.FIELD_TYPE_VAR_STRING:
          return "VARCHAR";
        case gweMysqlMetaResult.FIELD_TYPE_STRING:
          return "CHAR";
        case gweMysqlMetaResult.FIELD_TYPE_ENUM:
          return "ENUM";
        case gweMysqlMetaResult.FIELD_TYPE_SET:
          return "SET";
        case gweMysqlMetaResult.FIELD_TYPE_YEAR:
          return "YEAR"; // YEAR support!!
        default:
          return "UNKNOWN";
      }
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }



  public bool isReadOnly(int column) {
    return false;
  }

  public bool isWritable(int column) {
    return true;
  }

  public bool isDefinitelyWritable(int column) {
    return true;
  }

  //--------------------------JDBC 2.0-----------------------------------

  /**
   * <p>Returns the fully-qualified name of the Java class whose instances
   * are manufactured if the method <code>ResultSet.getObject</code>
   * is called to retrieve a value
   * from the column.  <code>ResultSet.getObject</code> may return a subclass of the
   * class returned by this method.
   *
   * @return the fully-qualified name of the class in the Java programming
   *         language that would be used by the method
   * <code>ResultSet.getObject</code> to retrieve the value in the specified
   * column. This is the class name used for custom mapping.
   * @exception SQLException if a database access error occurs
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public String getColumnClassName(int column) {
//  throw new SQLException("mysql does not support getColumnClassName(int).");
    try {
//      gweMysqlColumn col = mysql.getColumn(column);
      gweMysqlColumn col = cols[column-1];
      switch (col.getType()) {
        case gweMysqlMetaResult.FIELD_TYPE_DECIMAL:
          return "System.Decimal";
        case gweMysqlMetaResult.FIELD_TYPE_TINY:
          return "System.SByte";
        case gweMysqlMetaResult.FIELD_TYPE_SHORT:
          return "System.Int32";
        case gweMysqlMetaResult.FIELD_TYPE_LONG:
          return "System.Int64";
        case gweMysqlMetaResult.FIELD_TYPE_FLOAT:
          return "System.Single";
        case gweMysqlMetaResult.FIELD_TYPE_DOUBLE:
          return "System.Double";
        case gweMysqlMetaResult.FIELD_TYPE_NULL:
          return "null";
        case gweMysqlMetaResult.FIELD_TYPE_TIMESTAMP:
          return "System.DateTime";
        case gweMysqlMetaResult.FIELD_TYPE_LONGLONG:
          return "System.Int64";
        case gweMysqlMetaResult.FIELD_TYPE_INT24:
          return "System.Int32";
        case gweMysqlMetaResult.FIELD_TYPE_DATE:
          return "System.DateTime";
        case gweMysqlMetaResult.FIELD_TYPE_TIME:
          return "System.DateTime";
        case gweMysqlMetaResult.FIELD_TYPE_DATETIME:
          return "System.DateTime";
        case gweMysqlMetaResult.FIELD_TYPE_TINY_BLOB:
          return "byte[]"; //byte[].class.getName();
        case gweMysqlMetaResult.FIELD_TYPE_MEDIUM_BLOB:
          return "byte[]"; //byte[].class.getName();
        case gweMysqlMetaResult.FIELD_TYPE_LONG_BLOB:
          return "byte[]"; //byte[].class.getName();
        case gweMysqlMetaResult.FIELD_TYPE_BLOB:
//          return "byte[]"; //byte[].class.getName();
            // Thanks for INDEN-san
          if( col.isBinary() )
            return "byte[]";
          return "System.String";
        case gweMysqlMetaResult.FIELD_TYPE_VAR_STRING:
          return "System.String";
        case gweMysqlMetaResult.FIELD_TYPE_STRING:
          return "System.String";
        case gweMysqlMetaResult.FIELD_TYPE_ENUM:
          return "System.String";
        case gweMysqlMetaResult.FIELD_TYPE_SET:
          return "System.String";
        case gweMysqlMetaResult.FIELD_TYPE_YEAR: // YEAR support!!
          return "System.Int32";
        default:
          return "";
      }
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  //// for Serialization
  //private void writeObject(ObjectOutputStream oos) throws IOException {
  //  oos.defaultWriteObject();
  //}
  //private void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException {
  //  ois.defaultReadObject();
  //}
}

}
