//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 12.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

namespace gwe.sql {

using System;
using System.IO;
using System.Text;
using System.Collections;
using System.Collections.Specialized;
using System.Globalization;
using net.sql;

public class gweMysqlPreparedStatement : gweMysqlStatement, PreparedStatement {
  /**
   * A PreparedStatement
   */
  private string prepared_sql = null;
  /**
   * A Parameter Object Array
   */
  private object[] param_obj_array = null;
  /**
   * A Parameter Type Array
   */
  private int[] param_type_array = null;
  /**
   * characterEncoding
   */
  private string encoding;
  /**
   * nativeUnicodeMapper
   */
  private gweNativeUnicodeMapper mapperclass;
  /**
   * Add 0x5C(\) after 0x5C(\) when you execute PreparedStatement.setString
   */
  private bool add5CAfter5C;
  /**
   * Trace SQL Statement with LogWriter
   */
  private bool debugSQL;

  private static readonly string[] hex_table = {
    "00","01","02","03","04","05","06","07","08","09","0a","0b","0c","0d","0e","0f",
    "10","11","12","13","14","15","16","17","18","19","1a","1b","1c","1d","1e","1f",
    "20","21","22","23","24","25","26","27","28","29","2a","2b","2c","2d","2e","2f",
    "30","31","32","33","34","35","36","37","38","39","3a","3b","3c","3d","3e","3f",
    "40","41","42","43","44","45","46","47","48","49","4a","4b","4c","4d","4e","4f",
    "50","51","52","53","54","55","56","57","58","59","5a","5b","5c","5d","5e","5f",
    "60","61","62","63","64","65","66","67","68","69","6a","6b","6c","6d","6e","6f",
    "70","71","72","73","74","75","76","77","78","79","7a","7b","7c","7d","7e","7f",
    "80","81","82","83","84","85","86","87","88","89","8a","8b","8c","8d","8e","8f",
    "90","91","92","93","94","95","96","97","98","99","9a","9b","9c","9d","9e","9f",
    "a0","a1","a2","a3","a4","a5","a6","a7","a8","a9","aa","ab","ac","ad","ae","af",
    "b0","b1","b2","b3","b4","b5","b6","b7","b8","b9","ba","bb","bc","bd","be","bf",
    "c0","c1","c2","c3","c4","c5","c6","c7","c8","c9","ca","cb","cc","cd","ce","cf",
    "d0","d1","d2","d3","d4","d5","d6","d7","d8","d9","da","db","dc","dd","de","df",
    "e0","e1","e2","e3","e4","e5","e6","e7","e8","e9","ea","eb","ec","ed","ee","ef",
    "f0","f1","f2","f3","f4","f5","f6","f7","f8","f9","fa","fb","fc","fd","fe","ff",
  };

  /**
   * Constructs a new gweMysqlStatement object.
   * @param conn the gweMysqlConnection object
   * @param sql the PreparedStatement object
   * @param rs_type the result set type
   * @param rs_concur the result set concurrency
   */
  public gweMysqlPreparedStatement(gweMysqlConnection conn, String sql,
                                   int rs_type, int rs_concur,
                                   string enc, gweNativeUnicodeMapper mapper,
                                   bool add5C_mode,
                                   bool hasAutoGeneratedKey) :
                         base(conn, rs_type, rs_concur, hasAutoGeneratedKey) {
    prepared_sql = sql;
    encoding = enc;
    mapperclass = mapper;
    add5CAfter5C = add5C_mode;
    debugSQL = conn.isDebugSQL();

    int param_count = 0;
    StringTokenizer t_str = new StringTokenizer(prepared_sql, "?", true);
    while(t_str.hasMoreTokens())
    {
      string token = t_str.nextToken();
      if( token.Equals("?") )
        param_count++;
    }
    if( param_count > 0 )
    {
      param_obj_array  = new object[param_count];
      param_type_array = new int[param_count];
      int i;
      for(i = 0; i < param_obj_array.Length; i++)
        param_obj_array[i]  = null;
      for(i = 0; i < param_type_array.Length; i++)
        param_type_array[i] = Types.NULL;
    }
  }

  private string completeSQLString(){
    gweMysqlStringBuffer sql = new gweMysqlStringBuffer(1024);
    sql.append("");
    int param_index = -1;
    StringTokenizer t_str = new StringTokenizer(prepared_sql, "?", true);
    while(t_str.hasMoreTokens())
    {
      string token = t_str.nextToken();
      if( token.Equals("?") )
      {
        param_index++;
        object obj = param_obj_array[param_index];
        if( obj == null )
        {
          sql.append("NULL");
        }
        else
        {
          switch(param_type_array[param_index])
          {
            case Types.TINYINT:
              //sql.append(((Byte)obj).toString());
              sql.append(obj.ToString());
              break;
            case Types.SMALLINT:
              //sql.append(((Short)obj).toString());
              sql.append(obj.ToString());
              break;
            case Types.INTEGER:
              //sql.append(((Integer)obj).toString());
              sql.append(obj.ToString());
              break;
            case Types.BIGINT:
              //sql.append(((Long)obj).toString());
              sql.append(obj.ToString());
              break;
            case Types.FLOAT:
              sql.append("'");
              //sql.append(((Float)obj).toString());
              sql.append(obj.ToString());
              sql.append("'");
              break;
            case Types.DOUBLE:
              sql.append("'");
              //sql.append(((Double)obj).toString());
              sql.append(obj.ToString());
              sql.append("'");
              break;
            case Types.DECIMAL:
              sql.append("'");
              //sql.append(((BigDecimal)obj).toString());
              sql.append(obj.ToString());
              sql.append("'");
              break;
            case Types.VARCHAR:
            case Types.LONGVARCHAR: // Thanks for INDEN-san
              sql.append("'");
              //sql.append(convertQuoteAndBS((String)obj));
              sql.append(convertQuoteAndBS(obj.ToString()));
              sql.append("'");
              break;
            case Types.DATE:
              // ex. '1999-02-14'
              sql.append("'");
              //sql.append(((Date)obj).toString());
              sql.append(((DateTime)obj).ToString("yyyy-MM-dd"));
              sql.append("'");
              break;
            case Types.TIME:
              // ex. '12:34:56'
              sql.append("'");
              //sql.append(((Time)obj).toString());
              sql.append(((DateTime)obj).ToString("HH:mm:ss"));
              sql.append("'");
              break;
            case Types.TIMESTAMP:
              {
              // ex. '1999-02-14 12:34:56'
              sql.append("'");
              //String temp_timestamp = ((Timestamp)obj).toString();
              string temp_timestamp = ((DateTime)obj).ToString("yyyy-MM-dd HH:mm:ss");
              int dot_idx = temp_timestamp.IndexOf('.');
              if( dot_idx != -1 )
                sql.append(temp_timestamp.Substring(0, dot_idx-0));
              else
                sql.append(temp_timestamp);
              sql.append("'");
              }
              break;
            case Types.VARBINARY:
              {
              sql.append("0x");
              sbyte[] buf = (sbyte[])obj;
              for(int i = 0; i < buf.Length; i++)
              {
                int b = (int)buf[i];
                if( b < 0 )
                  b += 256;
                sql.append(gweMysqlPreparedStatement.hex_table[b]);
              }
              }
              break;
            case Types.LONGVARBINARY:
              {
              gweMysqlPreparedObject param_obj = (gweMysqlPreparedObject)obj;
              BinaryReader in_stream = (BinaryReader)param_obj.getObj();
              byte[] buf = null;
              int read_len = param_obj.getLength();

              try{
                buf = in_stream.ReadBytes(read_len);
              }
              catch(IOException io_e){
                ;
              }
              if( read_len > 0 )
              {
                sql.append("0x");
                for(int i = 0; i < read_len; i++)
                {
                  int b = (int)buf[i];
                  if( b < 0 )
                    b += 256;
                  sql.append(gweMysqlPreparedStatement.hex_table[b]);
                }
              }
              }
              break;
            default:
              break;
          }
        }
      }
      else
      {
        sql.append(token);
      }
    }
    return (sql.ToString());
  }

  private int getSQLStringLength(){
    int retval = 0;
    gweMysqlStringBuffer sql = new gweMysqlStringBuffer(1024);
    sql.append("");
    int param_index = -1;
    StringTokenizer t_str = new StringTokenizer(prepared_sql, "?", true);
    while(t_str.hasMoreTokens())
    {
      string token = t_str.nextToken();
      if( token.Equals("?") )
      {
        param_index++;
        object obj = param_obj_array[param_index];
        if( obj == null )
        {
          sql.append("NULL");
        }
        else
        {
          switch(param_type_array[param_index])
          {
            case Types.TINYINT:
              //sql.append(((Byte)obj).toString());
              sql.append(obj.ToString());
              break;
            case Types.SMALLINT:
              //sql.append(((Short)obj).toString());
              sql.append(obj.ToString());
              break;
            case Types.INTEGER:
              //sql.append(((Integer)obj).toString());
              sql.append(obj.ToString());
              break;
            case Types.BIGINT:
              //sql.append(((Long)obj).toString());
              sql.append(obj.ToString());
              break;
            case Types.FLOAT:
              sql.append("'");
              //sql.append(((Float)obj).toString());
              sql.append(obj.ToString());
              sql.append("'");
              break;
            case Types.DOUBLE:
              sql.append("'");
              //sql.append(((Double)obj).toString());
              sql.append(obj.ToString());
              sql.append("'");
              break;
            case Types.DECIMAL:
              sql.append("'");
              //sql.append(((BigDecimal)obj).toString());
              sql.append(obj.ToString());
              sql.append("'");
              break;
            case Types.VARCHAR:
            case Types.LONGVARCHAR: // Thanks for INDEN-san
              sql.append("'");
              //sql.append(convertQuoteAndBS((String)obj));
              sql.append(convertQuoteAndBS(obj.ToString()));
              sql.append("'");
              break;
            case Types.DATE:
              // ex. '1999-02-14'
              sql.append("'");
              //sql.append(((Date)obj).toString());
              sql.append(((DateTime)obj).ToString("yyyy-MM-dd"));
              sql.append("'");
              break;
            case Types.TIME:
              // ex. '12:34:56'
              sql.append("'");
              //sql.append(((Time)obj).toString());
              sql.append(((DateTime)obj).ToString("HH:mm:ss"));
              sql.append("'");
              break;
            case Types.TIMESTAMP:
              {
              // ex. '1999-02-14 12:34:56'
              sql.append("'");
              //String temp_timestamp = ((Timestamp)obj).toString();
              string temp_timestamp = ((DateTime)obj).ToString("yyyy-MM-dd HH:mm:ss");
              int dot_idx = temp_timestamp.IndexOf('.');
              if( dot_idx != -1 )
                sql.append(temp_timestamp.Substring(0, dot_idx-0));
              else
                sql.append(temp_timestamp);
              sql.append("'");
              }
              break;
            case Types.VARBINARY:
              {
              sql.append("0x");

              sbyte[] buf = (sbyte[])obj;
              retval += buf.Length * 2;
              }
              break;
            case Types.LONGVARBINARY:
              {
              sql.append("0x");

              gweMysqlPreparedObject param_obj = (gweMysqlPreparedObject)obj;
              retval += param_obj.getLength() * 2;
              }
              break;
            default:
              break;
          }
        }
      }
      else
      {
        sql.append(token);
      }
    }

    string temp_string = sql.ToString();
    try{
      byte[] temp_bytes;
      if( mapperclass != null )
      {
        char[] cbuf = temp_string.ToCharArray();
        temp_bytes = mapperclass.Unicode2Native(cbuf);
      }
      else
      {
        temp_bytes = Encoding.GetEncoding(encoding).GetBytes(temp_string);
      }
      retval += temp_bytes.Length;
    }
    catch(Exception e){
      retval += temp_string.Length;
    }
    return (retval);
  }

  public ResultSet executeQuery() {
    return base.executeQuery(completeSQLString());
  }

  public int executeUpdate() {
//  return super.executeUpdate(completeSQLString());

    bool writelog = debugSQL && (DriverManager.getLogWriter() != null);
    gweMysqlStringBuffer sqllog = null;

    try{
      // to avoid send new SQL statement before close old resultset!!
      connection.setCurResultSet(null);

      gweMysqlPacket packet = connection.getPacket();

      // Send query command and sql update
      packet.sendHeader(getSQLStringLength());

      if( writelog )
      {
        sqllog = new gweMysqlStringBuffer();
        System.DateTime t = System.DateTime.Now;
        sqllog.append(t.ToString() + ":");
      }

      int param_index = -1;
      StringTokenizer t_str = new StringTokenizer(prepared_sql, "?", true);
      while(t_str.hasMoreTokens())
      {
        string token = t_str.nextToken();
        if( token.Equals("?") )
        {
          param_index++;
          object obj = param_obj_array[param_index];
          if( obj == null )
          {
            packet.writeStringNoNullandSendBuffer("NULL");
            if( writelog ) sqllog.append("NULL");
          }
          else
          {
            switch(param_type_array[param_index])
            {
              case Types.TINYINT:
                //packet.writeStringNoNullandSendBuffer(((Byte)obj).toString());
                packet.writeStringNoNullandSendBuffer(obj.ToString());
                if( writelog ) sqllog.append(obj.ToString());
                break;
              case Types.SMALLINT:
                //packet.writeStringNoNullandSendBuffer(((Short)obj).toString());
                packet.writeStringNoNullandSendBuffer(obj.ToString());
                if( writelog ) sqllog.append(obj.ToString());
                break;
              case Types.INTEGER:
                //packet.writeStringNoNullandSendBuffer(((Integer)obj).toString());
                packet.writeStringNoNullandSendBuffer(obj.ToString());
                if( writelog ) sqllog.append(obj.ToString());
                break;
              case Types.BIGINT:
                //packet.writeStringNoNullandSendBuffer(((Long)obj).toString());
                packet.writeStringNoNullandSendBuffer(obj.ToString());
                if( writelog ) sqllog.append(obj.ToString());
                break;
              case Types.FLOAT:
                //packet.writeStringNoNullandSendBuffer("'" + ((Float)obj).toString() + "'");
                packet.writeStringNoNullandSendBuffer("'" + obj.ToString() + "'");
                if( writelog ) sqllog.append("'"+obj.ToString()+"'");
                break;
              case Types.DOUBLE:
                //packet.writeStringNoNullandSendBuffer("'" + ((Double)obj).toString() + "'");
                packet.writeStringNoNullandSendBuffer("'" + obj.ToString() + "'");
                if( writelog ) sqllog.append("'"+obj.ToString()+"'");
                break;
              case Types.DECIMAL:
                //packet.writeStringNoNullandSendBuffer("'" + ((BigDecimal)obj).toString() + "'");
                packet.writeStringNoNullandSendBuffer("'" + obj.ToString() + "'");
                if( writelog ) sqllog.append("'"+obj.ToString()+"'");
                break;
              case Types.VARCHAR:
              case Types.LONGVARCHAR: // Thanks for INDEN-san
                //packet.writeStringNoNullandSendBuffer("'" + convertQuoteAndBS((String)obj) + "'");
                packet.writeStringNoNullandSendBuffer("'" + convertQuoteAndBS(obj.ToString()) + "'");
                //if( writelog ) sqllog.append("'"+convertQuoteAndBS((String)obj)+"'");
                if( writelog ) sqllog.append("'"+convertQuoteAndBS(obj.ToString())+"'");
                break;
              case Types.DATE:
                // ex. '1999-02-14'
                //packet.writeStringNoNullandSendBuffer("'" + ((Date)obj).toString() + "'");
                packet.writeStringNoNullandSendBuffer("'" + ((DateTime)obj).ToString("yyyy-MM-dd") + "'");
                if( writelog ) sqllog.append("'"+((DateTime)obj).ToString("yyyy-MM-dd")+"'");
                break;
              case Types.TIME:
                // ex. '12:34:56'
                //packet.writeStringNoNullandSendBuffer("'" + ((Time)obj).toString() + "'");
                packet.writeStringNoNullandSendBuffer("'" + ((DateTime)obj).ToString("HH:mm:ss") + "'");
                if( writelog ) sqllog.append("'"+((DateTime)obj).ToString("HH:mm:ss")+"'");
                break;
              case Types.TIMESTAMP:
                {
                // ex. '1999-02-14 12:34:56'
                //String temp_timestamp = ((Timestamp)obj).toString();
                string temp_timestamp = ((DateTime)obj).ToString("yyyy-MM-dd HH:mm:ss");
                int dot_idx = temp_timestamp.IndexOf('.');
                if( dot_idx != -1 )
                  temp_timestamp = temp_timestamp.Substring(0, dot_idx-0);
                packet.writeStringNoNullandSendBuffer("'" + temp_timestamp + "'");
                if( writelog ) sqllog.append("'"+temp_timestamp+"'");
                }
                break;
              case Types.VARBINARY:
                {
                packet.writeStringNoNullandSendBuffer("0x");
                if( writelog ) sqllog.append("0x");
                
                sbyte[] buf = (sbyte[])obj;
                for(int i = 0; i < buf.Length; i++)
                {
                  int b = (int)buf[i];
                  if( b < 0 )
                    b += 256;
                  packet.writeStringNoNullandSendBufferEx(gweMysqlPreparedStatement.hex_table[b]);
                  if( writelog ) sqllog.append(gweMysqlPreparedStatement.hex_table[b]);
                }
                }
                break;
              case Types.LONGVARBINARY:
                {
                packet.writeStringNoNullandSendBuffer("0x");
                if( writelog ) sqllog.append("0x");
                
                gweMysqlPreparedObject param_obj = (gweMysqlPreparedObject)obj;
                BinaryReader in_stream = (BinaryReader)param_obj.getObj();
                byte[] buf = null;
                while(true)
                {
                  int read_len = -1;
                  try{
                    buf = in_stream.ReadBytes(4096);
                    read_len = buf.Length;
                  }
                  catch(IOException io_e){
                    throw new SQLException("mysql exception: " + io_e.Message);
                  }
                  if( read_len < 1 )
                    break;
                  for(int i = 0; i < read_len; i++)
                  {
                    int b = (int)buf[i];
                    if( b < 0 )
                      b += 256;
                    packet.writeStringNoNullandSendBufferEx(gweMysqlPreparedStatement.hex_table[b]);
                    if( writelog ) sqllog.append(gweMysqlPreparedStatement.hex_table[b]);
                  }
                }
                }
                break;
              default:
                break;
            }
          }
        }
        else
        {
          packet.writeStringNoNullandSendBuffer(token);
          if( writelog ) sqllog.append(token);
        }
      }

      packet.flushSendBuffer();

      if( writelog )
        DriverManager.println(sqllog.ToString());

      // check if we're ok
      packet.next();
      int updateStatus = packet.ub(packet.readByte());
      if (updateStatus==255) {
        string errorString = packet.readString();
        throw new SQLException("mysql exception: " + errorString);
      }
      long affectedRows = packet.readLength();
      long updateId = packet.readLength();

      autoIncrementValue = updateId;
      return (int)affectedRows;
    }
    catch( gweMysqlException e ) {
      throw new SQLException("mysql exception: " + e.Message);
    }
  }

  public void setNull(int parameterIndex, int sqlType) {
    this.setObject(parameterIndex, null, sqlType);
  }

  public void setBoolean(int parameterIndex, bool x) {
    throw new SQLException("mysql does not support setBoolean(int, boolean).");
  }

  public void setByte(int parameterIndex, sbyte x) {
    object obj = x;
    this.setObject(parameterIndex, obj, Types.TINYINT);
  }

  public void setShort(int parameterIndex, short x) {
    object obj = x;
    this.setObject(parameterIndex, obj, Types.SMALLINT);
  }

  public void setInt(int parameterIndex, int x) {
    object obj = x;
    this.setObject(parameterIndex, obj, Types.INTEGER);
  }

  public void setLong(int parameterIndex, long x) {
    object obj = x;
    this.setObject(parameterIndex, obj, Types.BIGINT);
  }

  public void setFloat(int parameterIndex, float x) {
    object obj = x;
    this.setObject(parameterIndex, obj, Types.FLOAT);
  }

  public void setDouble(int parameterIndex, double x) {
    object obj = x;
    this.setObject(parameterIndex, obj, Types.DOUBLE);
  }

  public void setBigDecimal(int parameterIndex, System.Decimal x) {
    this.setObject(parameterIndex, x, Types.DECIMAL);
  }

  /**
   * Add 0x5C(\) after 0x5C(\) when you execute PreparedStatement.setString
   */
  private byte[] add5CAfter5CforShiftJIS(byte[] buf) {
    int buflen = buf.Length;
    if( buflen == 0 )
      return buf;
    MemoryStream bos = new MemoryStream();
    int idx = 0;
    while(true)
    {
      int b1 = (int)buf[idx];
      if( b1 < 0 )
        b1 += 256;
      bos.WriteByte((byte)b1);
      if( (b1 >= 0x80 && b1 <= 0x9F) || (b1 >= 0xE0 && b1 <= 0xFC) )
      {
        if( idx < buflen-1 )
        {
          int b2 = (int)buf[idx+1];
          if( b2 < 0 )
            b2 += 256;
          if( b2 == 0x5C )
            bos.WriteByte((byte)b2);
        }
      }
      idx++;
      if( idx >= buflen )
        break;
    }
    return bos.ToArray();
  }

  public void setString(int parameterIndex, string x) {
    if( add5CAfter5C && x != null )
    {
      try {
        if( mapperclass != null )
        {
          char[] cbuf = x.ToCharArray();
          byte[] temp_bytes = mapperclass.Unicode2Native(cbuf);
          byte[] buf2 = add5CAfter5CforShiftJIS(temp_bytes);
          x = new string(mapperclass.Native2Unicode(buf2));
        }
        else
        {
          byte[] temp_bytes = Encoding.GetEncoding(encoding).GetBytes(x);
          byte[] buf2 = add5CAfter5CforShiftJIS(temp_bytes);
          x = Encoding.GetEncoding(encoding).GetString(buf2, 0, buf2.Length);
        }
      }
      catch( Exception e ) {
        throw new SQLException("mysql exception: " + e.Message);
      }
    }
    this.setObject(parameterIndex, x, Types.VARCHAR);
  }

  public void setBytes(int parameterIndex, sbyte[] x) {
    this.setObject(parameterIndex, x, Types.VARBINARY);
  }

  public void setDate(int parameterIndex, System.DateTime x) {
    // ex. '1999-02-14'
    this.setObject(parameterIndex, x, Types.DATE);
  }

  public void setTime(int parameterIndex, System.DateTime x) {
    // ex. '12:34:56'
    this.setObject(parameterIndex, x, Types.TIME);
  }

  public void setTimestamp(int parameterIndex, System.DateTime x) {
    // ex. '1999-02-14 12:34:56'
    this.setObject(parameterIndex, x, Types.TIMESTAMP);
  }

  public void setAsciiStream(int parameterIndex, BinaryReader x, int length) {
    throw new SQLException("mysql does not support setAsciiStream(int, InputStream, int).");
  }

  public void setUnicodeStream(int parameterIndex, BinaryReader x, int length) {
    throw new SQLException("mysql does not support setUnicodeStream(int, InputStream, int).");
  }

  public void setBinaryStream(int parameterIndex, BinaryReader x, int length) {
    this.setObject(parameterIndex, new gweMysqlPreparedObject(x, length), Types.LONGVARBINARY);
  }

  public void clearParameters() {
    if( param_obj_array  != null &&
        param_type_array != null )
    {
      int i;
      for(i = 0; i < param_obj_array.Length; i++)
        param_obj_array[i]  = null;
      for(i = 0; i < param_type_array.Length; i++)
        param_type_array[i] = Types.NULL;
    }
    else
    {
      throw new SQLException("mysql error : clearParameters(), no parameter.");
    }
  }

  public void setObject(int parameterIndex, object x, int targetSqlType, int scale) {
    this.setObject(parameterIndex, x, targetSqlType);
  }

  public void setObject(int parameterIndex, object x, int targetSqlType) {
    if( param_obj_array  != null &&
        param_type_array != null )
    {
      if( parameterIndex <= param_obj_array.Length  &&
          parameterIndex <= param_type_array.Length )
      {
        param_obj_array[parameterIndex-1]  = x;
        if( (x is sbyte[]) && (targetSqlType != Types.VARBINARY) )
          targetSqlType = Types.VARBINARY;
        if( (x is BinaryReader) && (targetSqlType != Types.LONGVARBINARY) )
          targetSqlType = Types.LONGVARBINARY;
        param_type_array[parameterIndex-1] = targetSqlType;
        return;
      }
    }
    throw new SQLException("gweMysqlPreparedStatement Error : setObject(int, Object, int).");
  }

  public void setObject(int parameterIndex, object x) {
    throw new SQLException("mysql does not support setObject(int, Object).");
  }

  public bool execute() {
    return base.execute(completeSQLString());
  }

  private string convertQuoteAndBS(string str) {
    gweMysqlStringBuffer retval = new gweMysqlStringBuffer(str.Length);
    retval.append("");
    StringTokenizer t_str = new StringTokenizer(str, "'\\", true);
    while(t_str.hasMoreTokens())
    {
      string token = t_str.nextToken();
      if( token.Equals("'") )
        retval.append("''");
      else if( token.Equals("\\") )
        retval.append("\\\\");
      else
        retval.append(token);
    }
    return (retval.ToString());
  }

  public new ResultSet executeQuery(string sql) {
    throw new SQLException("do not call this method : executeQuery(String)");
  }

  public new int executeUpdate(string sql) {
    throw new SQLException("do not call this method : executeUpdate(String)");
  }

  public new bool execute(string sql) {
    throw new SQLException("do not call this method : execute(String)");
  }

  //--------------------------JDBC 2.0-----------------------------------

  public void addBatch() {
    throw new SQLException("mysql does not support addBatch().");
  }

  public void setCharacterStream(int parameterIndex, StreamReader reader, int length) {
    throw new SQLException("mysql does not support setCharacterStream(int, Reader, int).");
  }

  public void setRef(int i, Ref x) {
    throw new SQLException("mysql does not support setRef(int, Ref).");
  }

  public void setBlob(int i, Blob x) {
    setBinaryStream(i, x.getBinaryStream(), (int)x.length());
  }

  public void setClob(int i, Clob x) {
    throw new SQLException("mysql does not support setClob(int, Clob).");
  }

  public void setArray(int i, net.sql.Array x) {
    throw new SQLException("mysql does not support setArray(int, Array).");
  }

  public ResultSetMetaData getMetaData() {
    ResultSet rs = getResultSet();
    return rs.getMetaData();
  }

  public void setDate(int parameterIndex, System.DateTime x, System.Globalization.Calendar cal) {
    // ex. '1999-02-14'
    this.setObject(parameterIndex, x, Types.DATE);
  }

  public void setTime(int parameterIndex, System.DateTime x, System.Globalization.Calendar cal) {
    // ex. '12:34:56'
    this.setObject(parameterIndex, x, Types.TIME);
  }

  public void setTimestamp(int parameterIndex, System.DateTime x, System.Globalization.Calendar cal) {
    // ex. '1999-02-14 12:34:56'
    this.setObject(parameterIndex, x, Types.TIMESTAMP);
  }

  public void setNull(int paramIndex, int sqlType, string typeName) {
    throw new SQLException("mysql does not support setNull(int, int, String).");
  }

  //------------------------- JDBC 3.0 -----------------------------------

  /**
   * Sets the designated parameter to the given <code>java.net.URL</code> value.
   * The driver converts this to an SQL <code>DATALINK</code> value
   * when it sends it to the database.
   *
   * @param parameterIndex the first parameter is 1, the second is 2, ...
   * @param x the <code>java.net.URL</code> object to be set
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void setURL(int parameterIndex, string x) {
    throw new SQLException("mysql does not support setURL(int, URL).");
  }
  /**
   * Retrieves the number, types and properties of this
   * <code>PreparedStatement</code> object's parameters.
   *
   * @return a <code>ParameterMetaData</code> object that contains information
   *         about the number, types and properties of this
   *         <code>PreparedStatement</code> object's parameters
   * @exception SQLException if a database access error occurs
   * @see ParameterMetaData
   * @since 1.4
   */
  public ParameterMetaData getParameterMetaData() {
    throw new SQLException("mysql does not support getParameterMetaData().");
  }
}

internal class gweMysqlPreparedObject {
  private int    length;
  private object obj;
  public gweMysqlPreparedObject(object x, int len) {
    length = len;
    obj    = x;
  }

  public int getLength() {
    return length;
  }

  public object getObj() {
    return obj;
  }
}

internal class gweMysqlStringBuffer {
  private char[] value;
  private int  count;

  public gweMysqlStringBuffer() : this(16) {
  }

  public gweMysqlStringBuffer(int length) {
    value = new char[length];
    count = 0;
  }

  public override string ToString() {
    return new string(value, 0, count);
  }

  public void append(string str) {
    if( str == null )
      return;
    int str_len = str.Length;
    if( str_len <= 0 )
      return;

    int newcount = count + str_len;
    if( newcount > value.Length )
      expandCapacity(newcount);
    char[] buff = str.ToCharArray(0, str_len);
    for(int i = 0; i < buff.Length; i++)
      value[count+i] = buff[i];
    count = newcount;
  }

  public void append(char c) {
    int newcount = count + 1;
    if( newcount > value.Length )
      expandCapacity(newcount);
    value[count] = c;
    count++;
  }

  private void expandCapacity(int minimumCapacity) {
    int newCapacity = (value.Length + 1) * 2;
    if( newCapacity < 0 )
    {
      newCapacity = Int32.MaxValue;
    }
    else if( minimumCapacity > newCapacity )
    {
      newCapacity = minimumCapacity;
    }
    char[] newValue = new char[newCapacity];
    for(int i = 0; i < count; i++)
      newValue[i] = value[i];
    value = newValue;
  }
}

internal class StringTokenizer {
  private int pos = 0;
  private StringCollection tokens = new StringCollection();
  public StringTokenizer(string str, string delim, bool returnDelims) {
    char[] sp = new char[delim.Length];
    for(int i = 0; i < delim.Length; i++)
      sp[i] = delim[i];
    ArrayList buff = new ArrayList();
    for(int i = 0; i < str.Length; i++)
    {
      char c = str[i];
      bool found = false;
      for(int j = 0; j < sp.Length; j++)
      {
        if( c == sp[j] )
        {
          found = true;
          break;
        }
      }
      if( !found )
      {
        buff.Add(c);
      }
      else
      {
        if( buff.Count > 0 )
        {
          object[] co = buff.ToArray();
          char[] cc = new char[co.Length];
          for(int k = 0; k < co.Length; k++)
            cc[k] = (char)co[k];
          string cs = new string(cc);
          tokens.Add(cs);
          buff.Clear();
        }
        if( returnDelims )
          tokens.Add(new string(new char[]{c}));
      }
    }
    if( buff.Count > 0 )
    {
      object[] co = buff.ToArray();
      char[] cc = new char[co.Length];
      for(int k = 0; k < co.Length; k++)
        cc[k] = (char)co[k];
      string cs = new string(cc);
      tokens.Add(cs);
      buff.Clear();
    }
  }

  public bool hasMoreTokens() {
    bool retval = false;
    int count = tokens.Count;
    pos++;
    if( pos >= 1 && pos <= count )
      retval = true;
    return retval;
  }

  public string nextToken() {
    return tokens[pos-1];
  }
}

}
