//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 16.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

namespace gwe.sql {

using System;
using System.IO;
using System.Text;
using System.Collections;
using System.Globalization;
using net.sql;

/**
 * This class implements the JDBC draft specification for the ResulSet
 * interface for mysql.  The class is designed to hold the results of
 * database queries.
 */
public class gweMysqlResultSetCore : net.sql.ResultSet {
  private gweMysql result;
  /**
   * The meta data for this result set.
   */
  private gweMysqlMetaResult meta;
  private Hashtable columnName;
  private Hashtable columnFullName;
  /**
   * Keeps track of the last column read.
   */
  private int last_read = -1;
  /**
   * result set type
   */
  private int resultSetType;
  /**
   * result set concurrency
   */
  private int resultSetConcurrency;
  /**
   * a hint as to the direction in which the rows in a result set
   */
  private int fetchDirection;
  /**
   * a hint as to the number of rows that should
   * be fetched from the database when more rows are needed.
   */
  private int fetchSize;
  /**
   * the Statement that produced this ResultSet object
   */
  private Statement statement;

  private bool packet_done = false;

  /**
   * Constructs a new result set object given the gweMysqlResult specified.
   * @param res the gweMysql returned from a previously executed query
   * @param rs_type the result set type
   * @param rs_concur the result set concurrency
   */
  public gweMysqlResultSetCore(gweMysql res, Statement st, int rs_type, int rs_concur) {
    result = res;
    statement = st;
    resultSetType = rs_type;
    resultSetConcurrency = rs_concur;
    fetchDirection = (int)ResultSet_FETCH.UNKNOWN;
    fetchSize = 0;

    meta = new gweMysqlMetaResult(result);
    columnName = (Hashtable)result.columnName.Clone();
    columnFullName = (Hashtable)result.columnFullName.Clone();
  }

  /**
   * JDBC draft specification method for moving the current row to the
   * next row, returning true if there is another row for processing.
   * @see java.sql.ResultSet#next
   * @exception SQLException thrown if an error occurs during processing
   * @return true if there are more rows to process, otherwise false
   */
  public bool next() {
    lock(this) {
      if( packet_done == true )
        return false;
      try {
        last_read=-1;
        bool retval = result.nextResult();
        if( retval == false )
          packet_done = true;
        return retval;
      }
      catch( Exception e ) {
        throw new SQLException(e.Message);
      }
    }
  }

  /**
   * JDBC specification method to determine if a column is null.
   * @see java.sql.ResultSet#isNull
   * @exception SQLException in the event of an gweMysqlException
   * @param column the column being tested for nullness
   * @return true if the column is null, false otherwise
   */
  public bool isNull(int column) {
    try {
      return (result.isNull(column));
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * JDBC specification method to determine if the last column read was null.
   * @see java.sql.ResultSet#wasNull
   * @exception SQLException in the event of an gweMysqlException
   * @return true if the column was null, false otherwise
   */
  public bool wasNull() {
    if( last_read == -1 )
      return false;
    return isNull(last_read);
  }

  /**
   * JDBC draft specification method for getting a string value from
   * the named column.  Note that the JDBC draft provides that this
   * method gets the value as a char, so you can retrieve int values
   * into String objects.
   * @see java.sql.ResultSet#getString
   * @exception SQLException thrown for invalid columns or bad rows
   * @param column the column being retrieved
   * @return the column as a String
   */
  public string getString(int column) {
    last_read = column;
    if( isNull(column) )
      return null;
    try {
//    return result.getString(column);

      try{
        byte[] b = result.getUBytes(column);
        int b_len = b.Length;
        string enc = result.getEncoding();
        gweNativeUnicodeMapper mapperclass = result.getNativeUnicodeMapper();
        string retval = "";
        while( b_len > 0 )
        {
          if( mapperclass != null )
            retval = new string(mapperclass.Native2Unicode(b));
          else
            retval = Encoding.GetEncoding(enc).GetString(b, 0, b_len);
          if( retval.Length > 0 )
            break;
          b_len--;
        }
        return retval;
      }
      catch(Exception enc_e){
        ;
      }
      return result.getString(column);
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * Get the value of a column in the current row as a Java byte.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL the result is 0
   */
  public sbyte getByte(int column) {
    string str;

    last_read = column;
    if( isNull(column) )
      return 0;
    try {
      str = result.getString(column);
      if( str.Equals("") )
        return 0;
      return (sbyte)Int32.Parse(str);
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * JDBC specification method for retrieving a column as a bool
   * value.  Interprets "", null, and "0" as false, others as true.
   * @see java.sql.ResultSet#getBoolean
   * @exception SQLException a sure sign of the apocolypse
   * @param column the column for which the value is being retrieved
   * @return false for "", null, or "0"; true otherwise
   */
  public bool getBoolean(int column) {
    last_read = column;
    if( isNull(column) )
      return false;
    try {
      string str = result.getString(column);

      if( str.Equals("") ) return false;
      if( str.Equals("0") ) return false;
      if( str.Equals("N") ) return false;
      if( str.Equals("n") ) return false;
      return true;
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * JDBC draft specification method to retrieve a short value from
   * the database.
   * @see java.sql.ResultSet#getShort
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @return the named column as a short
   */
  public short getShort(int column) {
    try {
      string str;
      last_read=column;
      str = result.getString(column);
      if( str == null )
        return 0;
      return (short)Int32.Parse(str);
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * JDBC draft specification method to retrieve an integer value from
   * the database.
   * @see java.sql.ResultSet#getInt
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @return the named column as an integer
   */
  public int getInt(int column) {
    try {
      string str;
      last_read=column;
      str = result.getString(column);
      if( str == null )
        return 0;
      return Int32.Parse(str);
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * JDBC draft specification method to retrieve a long value from
   * the database.
   * @see java.sql.ResultSet#getLong
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @return the named column as a short
   */
  public long getLong(int column) {
    try {
      string str;
      last_read=column;
      str = result.getString(column);
      if( str == null )
        return 0;
      return Int64.Parse(str);
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * JDBC draft specification method to retrieve a float value from
   * the database.
   * @see java.sql.ResultSet#getFloat
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @return the named column as a float
   */
  public float getFloat(int column) {
    try {
      string str;
      last_read=column;
      str = result.getString(column);
      if( str == null )
        return 0;
      return Single.Parse(str);
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * JDBC draft specification method to retrieve a double value from
   * the database.
   * @see java.sql.ResultSet#getDouble
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @return the named column as a double
   */
  public double getDouble(int column) {
    try {
      string str;
      last_read=column;
      str = result.getString(column);
      if( str == null )
        return 0;
      return Double.Parse(str);
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }


  /* @deprecated */
  public System.Decimal getBigDecimal(int column, int scale) {
    try {
      string str;
      last_read=column;
      str = result.getString(column);
      if( str == null )
        return System.Decimal.Parse("0"); // scale is ignored!!
      else
        return System.Decimal.Parse(str); // scale is ignored!!
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }


  /* @deprecated */
  public System.Decimal getBigDecimal(string columnName, int scale) {
    return getBigDecimal(findColumn(columnName), scale);
  }


  /**
   * JDBC draft specification method to retrieve a Bignum object from
   * the database.
   * @see java.sql.ResultSet#getBignum
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @param scale how many decimal digits after the floating point to maintain
   * @return the named column as a Bignum
  public Bignum getBignum(int column, int scale) {
    try {
    String str;
    last_read = column;
    str = result.getString(column);
    if( str == null ) return new Bignum(0, scale);
    else return new Bignum(str, scale);
  }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }
   */


  /**
   * JDBC draft specification method to retrieve a decimal from
   * the database.
   * @see java.sql.ResultSet#getDecimal
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @param scale how many decimal digits after the floating point to maintain
   * @return the named column as a Bignum
  public Bignum getDecimal(int column, int scale) {
    try {
    String str;
    last_read = column;
    str = result.getString(column);
    if( str == null ) return new Bignum(0, scale);
    else return new Bignum(str, scale);
  }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }
   */


  /**
   * JDBC draft specification method to return a byte array.
   * @see java.sql.ResultSet#getBytes
   * @exception SQLException thrown if something goes wrong
   * @param column the column being retrieved
   * @return a byte array that is the value of the column
   */

  public sbyte[] getSBytes(int column) {
    try {
      last_read = column;
      return result.getSBytes(column);
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }
  public byte[] getUBytes(int column) {
    try {
      last_read = column;
      return result.getUBytes(column);
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * JDBC 1.10 specification for retrieving a date column.
   * Can you say namespace pollution?  I knew you could.
   * @see java.sqlResultSet#getDate
   * @exception SQLException thrown in the event of problems
   * @param column the column being retrieved
   * @return the date value for the column
   */
  public System.DateTime getDate(int column) {
    try {
      string str;
      last_read = column;
      str = result.getString(column);
      if( str == null )
        return System.DateTime.MinValue;
      /*
      Integer y = new Integer(str.substring(0,2));
      Integer m = new Integer(str.substring(2,4));
      Integer d = new Integer(str.substring(4,6));
      return new java.sql.Date(y.intValue(), m.intValue()-1, d.intValue());
      */
      DateTimeFormatInfo format = new DateTimeFormatInfo();
      format.DateSeparator = "-";
      format.TimeSeparator = ":";
      return DateTime.Parse(str, format);
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * JDBC draft specification method for retrieving a time from the database.
   * @see java.sql.ResultSet#getTime
   * @exception SQLException thrown in the event of troubles
   * @param column the column being retrieved
   * @return the column as a java.sql.Time object
   */
  public System.DateTime getTime(int column) {
    try {
      string str;
      last_read = column;
      str = result.getString(column);
      if( str == null )
        return System.DateTime.MinValue;
      /*
      Integer h = new Integer(str.substring(6,8));
      Integer m = new Integer(str.substring(8,10));
      Integer s = new Integer(str.substring(10,12));
      return new java.sql.Time(h.intValue(), m.intValue(), s.intValue());
      */
      DateTimeFormatInfo format = new DateTimeFormatInfo();
      format.DateSeparator = "-";
      format.TimeSeparator = ":";
      return DateTime.Parse(str, format);
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * JDBC draft specification method for retrieving a timestamp from
   * the database.
   * @see java.sql.ResultSet#getTimestamp
   * @exception SQLException thrown in the event of troubles
   * @param column the column being retrieved
   * @return the column as a java.sql.Timestamp object
   */
  public System.DateTime getTimestamp(int column) {
    try {
      last_read = column;
      string str = result.getString(column);
      if( str == null )
        return System.DateTime.MinValue;

      DateTimeFormatInfo format = new DateTimeFormatInfo();
      format.DateSeparator = "-";
      format.TimeSeparator = ":";
      try {
        return System.DateTime.Parse(str, format);
      }
      catch(Exception e) {
        // Thanks for INDEN-san
        if( str.Length == 14 )
        {
          int y  = Int32.Parse(str.Substring(0,4-0));
          int m  = Int32.Parse(str.Substring(4,6-4));
          int d  = Int32.Parse(str.Substring(6,8-6));
          int hr = Int32.Parse(str.Substring(8,10-8));
          int mi = Int32.Parse(str.Substring(10,12-10));
          int se = Int32.Parse(str.Substring(12,14-12));
          return new DateTime(y, m, d, hr, mi, se, 0);
        }
        else if( str.Length == 12 )
        {
          int y  = Int32.Parse(str.Substring(0,2-0));
          int m  = Int32.Parse(str.Substring(2,4-2));
          int d  = Int32.Parse(str.Substring(4,6-4));
          int hr = Int32.Parse(str.Substring(6,8-6));
          int mi = Int32.Parse(str.Substring(8,10-8));
          int se = Int32.Parse(str.Substring(10,12-10));
          int two_digit_y = y;
          if( two_digit_y >= 0 && two_digit_y <= 69 )
            two_digit_y += 100;
          return new DateTime(two_digit_y, m, d, hr, mi, se, 0);
        }
        else
        {
          throw new SQLException(e.Message);
        }
      }
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * This is not currently supported.
   */
  public System.IO.BinaryReader getAsciiStream(int column) {
    throw new SQLException("mysql does not support getAsciiStream(int).");
  }

  /**
   * This is not currently supported.
   */
  public System.IO.BinaryReader getUnicodeStream(int column) {
    throw new SQLException("mysql does not support getUnicodeStream(int).");
  }

  /**
   * This is not currently supported.
   */
  public System.IO.BinaryReader getBinaryStream(int column) {
    try{
      byte[] bin_buf = getUBytes(column); // getUBytes() is for only C#!!
      if( bin_buf == null )
        return null;
      return new System.IO.BinaryReader(new System.IO.MemoryStream(bin_buf));
    }
    catch(Exception e) {
      throw new SQLException(e.Message);
    }
//    return null;
  }

  /**
   * JDBC draft specification method for closing a result set.
   * @see java.sql.ResultSet#close
   */
  public void close() {
    // We need to halt the flow of result packets -
    // for now, close the socket, and re-conect.
    try {
//    result.reConnect();
      if( packet_done == false )
      {
        while(true)
        {
          if( this.next() == false )
            break;
        }
      }
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * JDBC draft specification method for returning a cursor name.
   * @see java.sql.ResultSet#getCursorName
   * @return ""
   */
  public string getCursorName() {
    return "";
  }

  /**
   * JDBC draft specification method for returning meta-deta on a result
   * set.
   * @see java.sql.ResultSet#getMetaData
   * @exception SQLException thrown on error getting meta-data
   * @return ResultSetMetaData object containing result set info
   */
  public ResultSetMetaData getMetaData() {
//    if( meta == null ) {
//      meta = new gweMysqlMetaResult(result);
//    }
    return meta;
  }

  /**
   * JDBC draft specification method for retrieving data as objects.
   * @see java.sql.ResultSet#getObject
   * @exception SQLException in the event of retrieval! error
   * @param column the column desired
   * @param type the SQL data type of the field
   * @scale preceision for Bignums
   * @return the column specified as an Object
   */
  public object getObject(int column, int type, int scale) {
    switch(type) {
    case Types.BIT:
      return getBoolean(column);

    case Types.TINYINT:
      return (sbyte)getInt(column);

    case Types.SMALLINT:
      return getShort(column);

    case Types.INTEGER:
      return getInt(column);

    case Types.BIGINT:
      return getLong(column);

    case Types.FLOAT:
      return getFloat(column);

    case Types.REAL:
      return getFloat(column);

    case Types.DOUBLE:
      return getDouble(column);

    case Types.NUMERIC:
      return getBigDecimal(column, scale);

    case Types.DECIMAL:
      return getBigDecimal(column, scale);

    case Types.CHAR:
      return getString(column);

    case Types.VARCHAR:
      return getString(column);

    case Types.LONGVARCHAR:
      return getString(column);

    case Types.DATE:
      return getDate(column);

    case Types.TIME:
      return getTime(column);

    case Types.TIMESTAMP:
      return getTimestamp(column);

    case Types.BINARY:
      return getUBytes(column);

    case Types.VARBINARY:
      return getUBytes(column);

    case Types.LONGVARBINARY:
      return getUBytes(column);

    default:
      return null;
    }
  }

  /**
   * Same as above, except with a default scale to 10.
   */
  public object getObject(int column, int type) {
    return getObject(column, type, 10);
  }

  /**
   * Same as above, except using the column's default SQL type.
   */
  public object getObject(int column) {
    ResultSetMetaData meta = getMetaData();
    int type = meta.getColumnType(column);

    return getObject(column, type);
  }

  /**
   * JDBC 0.65 specification method for retrieving a column value
   * as a char based on the column name.
   * @see java.sql.ResultSet#getString
   * @param name the name of the column desired
   * @return the value of the column as a char
   */
  public string getString(string name) {
    return getString(findColumn(name));
  }

  /**
   * Returns the column as a byte based on column name
   */
  public sbyte getByte(string columnName) {
    return getByte(findColumn(columnName));
  }

  /**
   * Get the value of a BIT column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is false
   */
  public bool getBoolean(string columnName) {
    return getBoolean(findColumn(columnName));
  }

  /**
   * Get the value of a short by column name
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is 0
   */
  public short getShort(string columnName) {
    return getShort(findColumn(columnName));
  }

  /**
   * Get the value of a INTEGER column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is 0
   */
  public int getInt(string columnName) {
    return getInt(findColumn(columnName));
  }

  /**
   * Get the value of a long column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is 0
   */
  public long getLong(string columnName) {
    return getLong(findColumn(columnName));
  }

  /**
   * Get the value of a FLOAT column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is 0
   */
  public float getFloat(string columnName) {
    return getFloat(findColumn(columnName));
  }

  /**
   * Get the value of a DOUBLE column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is 0
   */
  public double getDouble(string columnName) {
    return getDouble(findColumn(columnName));
  }


  /**
   * Get the value of a NUMERIC column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is null

  public Bignum getBignum(String columnName, int scale) {
    return getBignum(findColumn(columnName), scale);
  }
   */


  /**
   * Get the value of a BINARY column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is null
   */
  public sbyte[] getSBytes(string columnName) {
    return getSBytes(findColumn(columnName));
  }
  public byte[] getUBytes(string columnName) {
    return getUBytes(findColumn(columnName));
  }

  /**
   * Get the value of a DATE column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is null
   */
  public System.DateTime getDate(string columnName) {
    return getDate(findColumn(columnName));
  }

  /**
   * Get the value of a TIME column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is null
   */
  public System.DateTime getTime(string columnName) {
    return getTime(findColumn(columnName));
  }

  /**
   * Get the value of a TIMESTAMP column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is null
   */
  public System.DateTime getTimestamp(string columnName) {
    return getTimestamp(findColumn(columnName));
  }

  /**
   * Very large ascii values in a LONGVARCHAR column can
   * be read in small chunks from a java.io.InputStream.
   *
   * <P><B>Note:</B> All the data in the returned stream must
   * be read prior to getting the value of any other column. The
   * next call to a get method implicitly closes the stream.
   *
   * @param columnName is the SQL name of the column
   * @return the java input stream which contains the ascii value
   */
  public System.IO.BinaryReader getAsciiStream(string columnName) {
    return getAsciiStream(findColumn(columnName));
  }

  /**
   * Very large unicode values in a LONGVARCHAR column can
   * be read in small chunks from a java.io.InputStream.
   *
   * <P><B>Note:</B> All the data in the returned stream must
   * be read prior to getting the value of any other column. The
   * next call to a get method implicitly closes the stream.
   *
   * @param columnName is the SQL name of the column
   * @return the java input stream which contains the unicode value
   */
  public System.IO.BinaryReader getUnicodeStream(string columnName) {
    return getUnicodeStream(findColumn(columnName));
  }

  /**
   * Very large binary values in a LONGVARBINARY column can
   * be read in small chunks from a java.io.InputStream.
   *
   * <P><B>Note:</B> All the data in the returned stream must
   * be read prior to getting the value of any other column. The
   * next call to a get method implicitly closes the stream.
   *
   * @param columnName is the SQL name of the column
   * @return the java input stream which contains the binary value */
  public System.IO.BinaryReader getBinaryStream(string columnName) {
    return getBinaryStream(findColumn(columnName));
  }

  /**
   * Get the value of a column as an object; an integral value is
   * returned as its java.lang equivalent object; a LONGVARCHAR or
   * LONGVARBINARY value is returned as a java.io.InputStream.
   *
   * @param columnName the SQL column name
   * @param sqlType SQL type code defined by java.sql.Types
   * @return the parameter value; if isNull the value is null
   */
  public object getObject(string columnName, int sqlType, int scale) {
    return getObject(findColumn(columnName), sqlType, scale);
  }

  /**
   * Same as above, except defaulting scale to 0.
   */
  public object getObject(string columnName, int type) {
    return getObject(findColumn(columnName), type, 0);
  }

  /**
   * Same as above, except returning the default SQL type
   */
  public object getObject(string columnName) {
    return getObject(findColumn(columnName));
  }

  /**
   * column name counter part for other isNull()
   */
  public bool isNull(string name) {
    return isNull(findColumn(name));
  }

  /**
   * Given a column name, this method returns the column number for that
   * name.  Column name to number mappings are kept inside a Hashtable.
   * Applications that do not need the overhead of this calculation are
   * not penalized since the mapping only occurs on the first attempt to
   * access a column number by name.
   * @exception java.sql.SQLException thrown if a bad name is passed
   * @param name the name of the column desired
   * @return the column number, 1 being the first column
   */
  public int findColumn(string name) {
    try {
//      return result.getColumnNum(name);
      object nI = columnName[name];
      if (nI==null)
        nI = columnFullName[name];
      if (nI!=null)
        return ((int)nI)+1;
      throw new SQLException("Invalid column name : "+name);
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * JDBC draft specification for getting the chain of warnings for this
   * statement.
   * @see java.sql.Statement#getWarnings
   * @return the chain of warnings
   */
  public SQLWarning getWarnings() {
    return null;
  }

  /**
   * JDBC draft specification for clearing the warning chain.
   * @see java.sql.Statement#clearWarnings
   */
  public void clearWarnings() {
  }

  internal string getEncoding() {
    return result.getEncoding();
  }

  internal gweNativeUnicodeMapper getNativeUnicodeMapper() {
    return result.getNativeUnicodeMapper();
  }

  //--------------------------JDBC 2.0-----------------------------------

  public System.IO.StreamReader getCharacterStream(int columnIndex) {
    return null;
  }

  public System.IO.StreamReader getCharacterStream(string columnName) {
    return getCharacterStream(findColumn(columnName));
  }

  public System.Decimal getBigDecimal(int columnIndex) {
    return getBigDecimal(columnIndex, 10);
  }

  public System.Decimal getBigDecimal(string columnName) {
    return getBigDecimal(findColumn(columnName));
  }

  public bool isBeforeFirst() {
    throw new SQLException("mysql does not support isBeforeFirst().");
  }

  public bool isAfterLast() {
    throw new SQLException("mysql does not support isAfterLast().");
  }

  public bool isFirst() {
    throw new SQLException("mysql does not support isFirst().");
  }

  public bool isLast() {
    throw new SQLException("mysql does not support isLast().");
  }

  public void beforeFirst() {
    throw new SQLException("mysql does not support beforeFirst().");
  }

  public void afterLast() {
    throw new SQLException("mysql does not support afterLast().");
  }

  public bool first() {
    throw new SQLException("mysql does not support first().");
  }

  public bool last() {
    throw new SQLException("mysql does not support last().");
  }

  public int getRow() {
    return 0;
  }

  public bool absolute(int row) {
    throw new SQLException("mysql does not support absolute(int).");
  }

  public bool relative(int rows) {
    throw new SQLException("mysql does not support relative(int).");
  }

  public bool previous() {
    throw new SQLException("mysql does not support previous().");
  }

  public void setFetchDirection(int direction) {
    fetchDirection = direction;
  }

  public int getFetchDirection() {
    return fetchDirection;
  }

  public void setFetchSize(int rows) {
    fetchSize = rows;
  }

  public int getFetchSize() {
    return fetchSize;
  }

  public int getType() {
    return resultSetType;
  }

  public int getConcurrency() {
    return resultSetConcurrency;
  }

  public bool rowUpdated() {
    throw new SQLException("mysql does not support rowUpdated().");
  }

  public bool rowInserted() {
    throw new SQLException("mysql does not support rowInserted().");
  }

  public bool rowDeleted() {
    throw new SQLException("mysql does not support rowDeleted().");
  }

  public void updateNull(int columnIndex) {
    throw new SQLException("mysql does not support updateNull(int).");
  }

  public void updateBoolean(int columnIndex, bool x) {
    throw new SQLException("mysql does not support updateBoolean(int, bool).");
  }

  public void updateByte(int columnIndex, sbyte x) {
    throw new SQLException("mysql does not support updateByte(int, byte).");
  }

  public void updateShort(int columnIndex, short x) {
    throw new SQLException("mysql does not support updateShort(int, short).");
  }

  public void updateInt(int columnIndex, int x) {
    throw new SQLException("mysql does not support updateInt(int, int).");
  }

  public void updateLong(int columnIndex, long x) {
    throw new SQLException("mysql does not support updateLong(int, long).");
  }

  public void updateFloat(int columnIndex, float x) {
    throw new SQLException("mysql does not support updateFloat(int, float).");
  }

  public void updateDouble(int columnIndex, double x) {
    throw new SQLException("mysql does not support updateDouble(int, double).");
  }

  public void updateBigDecimal(int columnIndex, System.Decimal x) {
    throw new SQLException("mysql does not support updateBigDecimal(int, BigDecimal).");
  }

  public void updateString(int columnIndex, string x) {
    throw new SQLException("mysql does not support updateString(int, String).");
  }

  public void updateBytes(int columnIndex, sbyte[] x) {
    throw new SQLException("mysql does not support updateBytes(int, byte[]).");
  }

  public void updateDate(int columnIndex, System.DateTime x) {
    throw new SQLException("mysql does not support updateDate(int, Date).");
  }

  public void updateTime(int columnIndex, System.DateTime x) {
    throw new SQLException("mysql does not support updateTime(int, Time).");
  }

  public void updateTimestamp(int columnIndex, System.DateTime x) {
    throw new SQLException("mysql does not support updateTimestamp(int, Timestamp).");
  }

  public void updateAsciiStream(int columnIndex, System.IO.BinaryReader x, int length) {
    throw new SQLException("mysql does not support updateAsciiStream(int, InputStream, int).");
  }

  public void updateBinaryStream(int columnIndex, System.IO.BinaryReader x, int length) {
    throw new SQLException("mysql does not support updateBinaryStream(int, InputStream, int).");
  }

  public void updateCharacterStream(int columnIndex, System.IO.StreamReader x, int length) {
    throw new SQLException("mysql does not support updateCharacterStream(int, Reader, int).");
  }

  public void updateObject(int columnIndex, object x, int scale) {
    throw new SQLException("mysql does not support updateObject(int, Object, int).");
  }

  public void updateObject(int columnIndex, object x) {
    throw new SQLException("mysql does not support updateObject(int, Object).");
  }

  public void updateNull(string columnName) {
    throw new SQLException("mysql does not support updateNull(String).");
  }

  public void updateBoolean(string columnName, bool x) {
    throw new SQLException("mysql does not support updateBoolean(String, bool).");
  }

  public void updateByte(string columnName, sbyte x) {
    throw new SQLException("mysql does not support updateByte(String, byte).");
  }

  public void updateShort(string columnName, short x) {
    throw new SQLException("mysql does not support updateShort(String, short).");
  }

  public void updateInt(string columnName, int x) {
    throw new SQLException("mysql does not support updateInt(String, int).");
  }

  public void updateLong(string columnName, long x) {
    throw new SQLException("mysql does not support updateLong(String, long).");
  }

  public void updateFloat(string columnName, float x) {
    throw new SQLException("mysql does not support updateFloat(String, float).");
  }

  public void updateDouble(string columnName, double x) {
    throw new SQLException("mysql does not support updateDouble(String, double).");
  }

  public void updateBigDecimal(string columnName, System.Decimal x) {
    throw new SQLException("mysql does not support updateBigDecimal(String, BigDecimal).");
  }

  public void updateString(string columnName, string x) {
    throw new SQLException("mysql does not support updateString(String, String).");
  }

  public void updateBytes(string columnName, sbyte[] x) {
    throw new SQLException("mysql does not support updateBytes(String, byte[]).");
  }

  public void updateDate(string columnName, System.DateTime x) {
    throw new SQLException("mysql does not support updateDate(String, Date).");
  }

  public void updateTime(string columnName, System.DateTime x) {
    throw new SQLException("mysql does not support updateTime(String, Time).");
  }

  public void updateTimestamp(string columnName, System.DateTime x) {
    throw new SQLException("mysql does not support updateTimestamp(String, Timestamp).");
  }

  public void updateAsciiStream(string columnName, System.IO.BinaryReader x, int length) {
    throw new SQLException("mysql does not support updateAsciiStream(String, InputStream, int).");
  }

  public void updateBinaryStream(string columnName, System.IO.BinaryReader x, int length) {
    throw new SQLException("mysql does not support updateBinaryStream(String, InputStream, int).");
  }

  public void updateCharacterStream(string columnName, System.IO.StreamReader reader, int length) {
    throw new SQLException("mysql does not support updateCharacterStream(String, Reader, int).");
  }

  public void updateObject(string columnName, object x, int scale) {
    throw new SQLException("mysql does not support updateObject(String, Object, int).");
  }

  public void updateObject(string columnName, object x) {
    throw new SQLException("mysql does not support updateObject(String, Object).");
  }

  public void insertRow() {
    throw new SQLException("mysql does not support insertRow().");
  }

  public void updateRow() {
    throw new SQLException("mysql does not support updateRow().");
  }

  public void deleteRow() {
    throw new SQLException("mysql does not support deleteRow().");
  }

  public void refreshRow() {
    throw new SQLException("mysql does not support refreshRow().");
  }

  public void cancelRowUpdates() {
    throw new SQLException("mysql does not support cancelRowUpdates().");
  }

  public void moveToInsertRow() {
    throw new SQLException("mysql does not support moveToInsertRow().");
  }

  public void moveToCurrentRow() {
    throw new SQLException("mysql does not support moveToCurrentRow().");
  }

  public Statement getStatement() {
    return statement;
  }

  public object getObject(int i, System.Collections.Hashtable map) {
    throw new SQLException("mysql does not support getObject(int, Map).");
  }

  public Ref getRef(int i) {
    throw new SQLException("mysql does not support getRef(int).");
  }

  public Blob getBlob(int i) {
    throw new SQLException("mysql does not support getBlob(int).");
  }

  public Clob getClob(int i) {
    throw new SQLException("mysql does not support getClob(int).");
  }

  public net.sql.Array getArray(int i) {
    throw new SQLException("mysql does not support getArray(int).");
  }

  public object getObject(string colName, System.Collections.Hashtable map) {
    throw new SQLException("mysql does not support getObject(String, Map).");
  }

  public Ref getRef(string colName) {
    throw new SQLException("mysql does not support getRef(String).");
  }

  public Blob getBlob(string colName) {
    throw new SQLException("mysql does not support getBlob(String).");
  }

  public Clob getClob(string colName) {
    throw new SQLException("mysql does not support getClob(String).");
  }

  public net.sql.Array getArray(string colName) {
    throw new SQLException("mysql does not support getArray(String).");
  }

  public System.DateTime getDate(int columnIndex, System.Globalization.Calendar cal) {
    return getDate(columnIndex);
  }

  public System.DateTime getDate(string columnName, System.Globalization.Calendar cal) {
    return getDate(findColumn(columnName));
  }

  public System.DateTime getTime(int columnIndex, System.Globalization.Calendar cal) {
    return getTime(columnIndex);
  }

  public System.DateTime getTime(string columnName, System.Globalization.Calendar cal) {
    return getTime(findColumn(columnName));
  }

  public System.DateTime getTimestamp(int columnIndex, System.Globalization.Calendar cal) {
    return getTimestamp(columnIndex);
  }

  public System.DateTime getTimestamp(string columnName, System.Globalization.Calendar cal) {
    return getTimestamp(findColumn(columnName));
  }

  //-------------------------- JDBC 3.0 ----------------------------------------

  /**
   * Retrieves the value of the designated column in the current row
   * of this <code>ResultSet</code> object as a <code>java.net.URL</code>
   * object in the Java programming language.
   *
   * @param columnIndex the index of the column 1 is the first, 2 is the second,...
   * @return the column value as a <code>java.net.URL</code> object;
   * if the value is SQL <code>NULL</code>,
   * the value returned is <code>null</code> in the Java programming language
   * @exception SQLException if a database access error occurs,
   *            or if a URL is malformed
   * @since 1.4
   */
  public string getURL(int columnIndex) {
    throw new SQLException("mysql does not support getURL(int).");
  }
  /**
   * Retrieves the value of the designated column in the current row
   * of this <code>ResultSet</code> object as a <code>java.net.URL</code>
   * object in the Java programming language.
   *
   * @param columnName the SQL name of the column
   * @return the column value as a <code>java.net.URL</code> object;
   * if the value is SQL <code>NULL</code>,
   * the value returned is <code>null</code> in the Java programming language
   * @exception SQLException if a database access error occurs
   *            or if a URL is malformed
   * @since 1.4
   */
  public string getURL(string columnName) {
    throw new SQLException("mysql does not support getURL(String).");
  }
  /**
   * Updates the designated column with a <code>java.sql.Ref</code> value.
   * The updater methods are used to update column values in the
   * current row or the insert row.  The updater methods do not
   * update the underlying database; instead the <code>updateRow</code> or
   * <code>insertRow</code> methods are called to update the database.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @param x the new column value
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void updateRef(int columnIndex, Ref x) {
    throw new SQLException("mysql does not support updateRef(int, Ref).");
  }
  /**
   * Updates the designated column with a <code>java.sql.Ref</code> value.
   * The updater methods are used to update column values in the
   * current row or the insert row.  The updater methods do not
   * update the underlying database; instead the <code>updateRow</code> or
   * <code>insertRow</code> methods are called to update the database.
   *
   * @param columnName the name of the column
   * @param x the new column value
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void updateRef(string columnName, Ref x) {
    throw new SQLException("mysql does not support updateRef(String, Ref).");
  }
  /**
   * Updates the designated column with a <code>java.sql.Blob</code> value.
   * The updater methods are used to update column values in the
   * current row or the insert row.  The updater methods do not
   * update the underlying database; instead the <code>updateRow</code> or
   * <code>insertRow</code> methods are called to update the database.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @param x the new column value
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void updateBlob(int columnIndex, Blob x) {
    throw new SQLException("mysql does not support updateBlob(int, Blob).");
  }
  /**
   * Updates the designated column with a <code>java.sql.Blob</code> value.
   * The updater methods are used to update column values in the
   * current row or the insert row.  The updater methods do not
   * update the underlying database; instead the <code>updateRow</code> or
   * <code>insertRow</code> methods are called to update the database.
   *
   * @param columnName the name of the column
   * @param x the new column value
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void updateBlob(string columnName, Blob x) {
    throw new SQLException("mysql does not support updateBlob(String, Blob).");
  }
  /**
   * Updates the designated column with a <code>java.sql.Clob</code> value.
   * The updater methods are used to update column values in the
   * current row or the insert row.  The updater methods do not
   * update the underlying database; instead the <code>updateRow</code> or
   * <code>insertRow</code> methods are called to update the database.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @param x the new column value
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void updateClob(int columnIndex, Clob x) {
    throw new SQLException("mysql does not support updateClob(int, Blob).");
  }
  /**
   * Updates the designated column with a <code>java.sql.Clob</code> value.
   * The updater methods are used to update column values in the
   * current row or the insert row.  The updater methods do not
   * update the underlying database; instead the <code>updateRow</code> or
   * <code>insertRow</code> methods are called to update the database.
   *
   * @param columnName the name of the column
   * @param x the new column value
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void updateClob(string columnName, Clob x) {
    throw new SQLException("mysql does not support updateClob(String, Blob).");
  }
  /**
   * Updates the designated column with a <code>java.sql.Array</code> value.
   * The updater methods are used to update column values in the
   * current row or the insert row.  The updater methods do not
   * update the underlying database; instead the <code>updateRow</code> or
   * <code>insertRow</code> methods are called to update the database.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @param x the new column value
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void updateArray(int columnIndex, net.sql.Array x) {
    throw new SQLException("mysql does not support updateArray(int, Array).");
  }
  /**
   * Updates the designated column with a <code>java.sql.Array</code> value.
   * The updater methods are used to update column values in the
   * current row or the insert row.  The updater methods do not
   * update the underlying database; instead the <code>updateRow</code> or
   * <code>insertRow</code> methods are called to update the database.
   *
   * @param columnName the name of the column
   * @param x the new column value
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void updateArray(string columnName, net.sql.Array x) {
    throw new SQLException("mysql does not support updateArray(String, Array).");
  }
}

}
